var dictionarySwitcher = {
  dictMenu: {},
  init: function () {
    var Cc = Components.classes
    Cc["@mozilla.org/spellchecker/" + ("@mozilla.org/spellchecker/myspell;1" in Cc ? "myspell;1" : "engine;1")]
      .createInstance(mozISpellCheckingEngine)
      .getDictionaryList(this.list = {}, {});
    this.list = this.list.value.toString().split(",");
    if (this.list.length <= 1)
      return;

    this.panel = document.getElementById("dictionary-switcher");

    var menu = document.getElementById("dictionary-switcher-menu");
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("type", "radio");
    menuitem.setAttribute("name", "dictionary-switcher-dicts");
    menuitem.setAttribute("oncommand", "dictionarySwitcher.toggle(this.label)");
    for (var i = 0; i < this.list.length; i++) {
      menuitem = menuitem.cloneNode(true);
      menuitem.setAttribute("label", this.list[i]);
      this.dictMenu[this.list[i]] = menuitem;
      menu.appendChild (menuitem);
    }

    this.prefService.addObserver("spellchecker.dictionary", this, false);
    this.prefService.addObserver("extensions.dictionary-switcher.autodetect", this, false);
    // XXX 1.8
    var self = this;
    this._eventListener = function (event) { self.handleEvent(event); };
    GetCurrentEditorElement().addEventListener("load", this._eventListener, true);
    GetMsgSubjectElement().addEventListener("input", this._eventListener, false);

    this.autodetect = this.prefService.getBoolPref("extensions.dictionary-switcher.autodetect");
    this.dict = this.prefService.getCharPref("spellchecker.dictionary");
    this.updateUI(false);
  },
  uninit: function () {
    this.prefService.removeObserver("spellchecker.dictionary", this);
    gMsgCompose.editor.removeEditActionListener(this);
    GetCurrentEditorElement().removeEventListener("load", this._eventListener, true);
    GetMsgSubjectElement().removeEventListener("input", this._eventListener, false);
  },
  get prefService() {
    delete this.prefService;
    return this.prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                         .getService(Components.interfaces.nsIPrefBranch2);
  },
  get scCompose() {
    delete this.scCompose;
    return this.scCompose = gMsgCompose.editor.getInlineSpellChecker ?
                             gMsgCompose.editor.getInlineSpellChecker(true) :
                             gMsgCompose.editor.inlineSpellChecker;
  },
  get scSubject() {
    var editor = GetMsgSubjectElement().editor;
    delete this.scSubject;
    return this.scSubject = editor.getInlineSpellChecker ?
                             editor.getInlineSpellChecker(true) :
                             editor.inlineSpellChecker;
  },
  updateUI: function (recheck) {
    this.panel.label = this.dict;
    if (this.dictMenu._last)
      this.dictMenu._last.removeAttribute("checked");
    if (this.dictMenu._last = this.dictMenu[ this.dict ])
      this.dictMenu._last.setAttribute("checked", true);
    if (recheck) {
      this.scCompose.spellChecker.SetCurrentDictionary(this.dict);
      this.scCompose.spellCheckRange(null);
      this.scSubject.spellCheckRange(null);
    }
  },
  toggle: function (dict) {
    this.dict = dict || this.getNextInList(this.dict);
    this.updateUI(true);
    this.prefService.setCharPref("spellchecker.dictionary", this.dict);
  },
  getNextInList: function (dict) {
    var i = this.list.indexOf(dict) + 1;
    if (i >= this.list.length)
      i = 0;
    return this.list[i];
  },
  detectDictionary: function (inlineSpellChecker, text) {
    text = text.split(/[^\w\x80-\x9F\xA2-\xA4\xC0-\xD6\xD8-\xF6\xF9-\xFF]+/i);
    var l = text.length;
    var maxMisspelled = Math.round(l / 4);
    var sc = inlineSpellChecker.spellChecker;
    var dict = this.dict;
    var misspelled, oldMisspelled, newDict;
    do {
      misspelled = 0;
      if (dict != this.dict)
        sc.SetCurrentDictionary(dict);
      for (var i = 0; i < l; i++)
        if (sc.CheckCurrentWord(text[i]))
          misspelled++;
      if (!newDict || misspelled < oldMisspelled)
        newDict = dict;
      oldMisspelled = misspelled;
      dict = this.getNextInList(dict);
    } while (misspelled > maxMisspelled && dict != this.dict);
    if (newDict && newDict != this.dict)
      this.toggle(newDict);
    else
      sc.SetCurrentDictionary(this.dict);
  },

  /* nsIObserver */
  observe: function (subject, topic, data) {
    if (topic == "nsPref:changed") {
      switch(data) {
        case "spellchecker.dictionary":
          var dict = this.prefService.getCharPref("spellchecker.dictionary");
          if (dict != this.dict) {
            this.dict = dict;
            this.updateUI(false);
          }
          break;
        case "extensions.dictionary-switcher.autodetect":
          this.autodetect = this.prefService.getBoolPref("extensions.dictionary-switcher.autodetect");
          break;
      }
    }
  },

  /* nsIDOMEventListener */
  handleEvent: function (event) {
    switch (event.type) {
      case "input":
        if (this.autodetect) {
          var text = event.target.value;
          var l = text.length;
          if (l == 10 || l == 20)
            this.detectDictionary(this.scSubject, text);
        }
        break;
      case "load":
        gMsgCompose.editor.addEditActionListener(this);
        break;
    }
  },

  /* nsIEditActionListener */
  DidCreateNode: function(){},
  DidDeleteNode: function(){},
  DidDeleteSelection: function(){},
  DidDeleteText: function(){},
  DidInsertNode: function(){},
  DidInsertText: function(){},
  DidJoinNodes: function(){},
  DidSplitNode: function(){},
  WillCreateNode: function(){},
  WillDeleteNode: function(){},
  WillDeleteSelection: function(){},
  WillDeleteText: function(){},
  WillInsertNode: function(){},
  WillInsertText: function (textNode, offset, string) {
    if (this.autodetect) {
      var l = textNode.length;
      if (l == 15 || l == 30)
        this.detectDictionary(this.scCompose, textNode.textContent);
    }
  },
  WillJoinNodes: function(){},
  WillSplitNode: function(){}
};

window.addEventListener("load", function () {
  dictionarySwitcher.init();
}, false);

window.addEventListener("unload", function () {
  dictionarySwitcher.uninit();
}, false);
